/*
 * Meta, Geoffrey's self-balancing ride-on robot.
 * 
 * Copyright (C) 2006 Trevor Blackwell <tlb@tlb.org>
 * Copyright (C) 2006 Geoffrey D. Bennett <g@netcraft.com.au>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "uart.h"

void adc_init(void) {
  /* turn off analogue comparator as we don't use it */
  ACSR = (1 << ACD);

  /* select PA0 */
  ADMUX = 0;

  /* Set ADC prescaler to 64, enable ADC, and start conversion */
  ADCSRA = (1 << ADPS2) | (1 << ADPS1)
         | (1 << ADEN)
         | (1 << ADSC);

  /* wait until bogus first conversion finished */
  while (ADCSRA & (1 << ADSC)) {
  }
}

uint16_t adc_read(uint8_t channel) {
  /* select channel */
  ADMUX = channel;

  /* start conversion */
  ADCSRA |= (1 << ADSC);

  /* wait until conversion finished */
  while (ADCSRA & (1 << ADSC)) {
  }

  /* return the result */
  return ADCW;
}
