/*
 * Meta, Geoffrey's self-balancing ride-on robot.
 * 
 * Copyright (C) 2006 Trevor Blackwell <tlb@tlb.org>
 * Copyright (C) 2006-2007 Geoffrey D. Bennett <g@netcraft.com.au>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef _CONFIG_H_
#define _CONFIG_H_

#define EEPROM_VERSION 0x01

struct config_struct {
  float pitch_rate_conv;
  float ay_conv;
  float ay_bal_offset;
  float batt_conv;

  float p_gain;
  float d_gain;
  float i_gain;

  float hard_speed_lim;
  float speed_limit_fwd;
  float speed_limit_rev;
  float overspeed_p_gain;
  float overspeed_i_gain;
  float not_overspeed_i_gain;
  float overspeed_angle_limit;

  float softstart_delay;
  float tip_angle;

  float turn_range;
  float turn_gain;
  float turn_backoff;
  float turn_filter;

  float rc_throttle_midpoint;
  float rc_throttle_range;
  float rc_throttle_angle_conv;
  float rc_throttle_filter;

  float rc_steer_midpoint;
  float rc_steer_range;
  float rc_steer_conv;

  float motor_l_gain;
  float motor_r_gain;
};

#define NUM_CONFIG_VAR (sizeof(struct config_struct) / sizeof(float))

/* length of config_struct in words, for calculate_checksum() */
#define CONFIG_WORD_LEN sizeof(struct config_struct) / sizeof(uint16_t)

extern const struct config_struct default_config;

extern union config_union {
  /* struct for named access */
  struct config_struct s;

  /* float array for remote programming by index number */
  float a[NUM_CONFIG_VAR];

  /* int array for calculating checksum */
  uint16_t c[CONFIG_WORD_LEN];
} config;

void config_load_defaults(void);
void eeprom_save_config(void);
int eeprom_load_config(void);

#endif
