/*
 * Meta, Geoffrey's self-balancing ride-on robot.
 * 
 * Copyright (C) 2006-2007 Geoffrey D. Bennett <g@netcraft.com.au>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <avr/io.h>

#include "motor.h"
#include "hardware.h"

#define LEFT_DISABLE() PORTD |= (1 << PD7)
#define LEFT_ENABLE() PORTD &= ~(1 << PD7)
#define RIGHT_DISABLE() PORTC |= (1 << PC3)
#define RIGHT_ENABLE() PORTC &= ~(1 << PC3)

int set_motors(float value_l, float value_r) {

  /* convert from -1..1 to 1..255 */
  int ivalue_l = value_l * 127 + 128;
  int ivalue_r = value_r * 127 + 128;

  if (ivalue_l < 0) {
    ivalue_l = 0;
  } else if (ivalue_l > 255) {
    ivalue_l = 255;
  }

  if (ivalue_r < 0) {
    ivalue_r = 0;
  } else if (ivalue_r > 255) {
    ivalue_r = 255;
  }

  OCR1AL = ivalue_l;
  OCR1BL = ivalue_r;
}

void init_motor(void) {
  LEFT_DISABLE();
  RIGHT_DISABLE();

  /* PWM setup */
  TCCR1A = TCCR1A_INIT;
  TCCR1B = TCCR1B_INIT;

  /* clear PC2 (mode-select) to select locked antiphase */
  PORTC &= ~(1 << PC2);
  set_motors(0.0, 0.0);

  LEFT_ENABLE();
  RIGHT_ENABLE();
}
